#include "contentssetter.h"

#include <QHBoxLayout>
#include <QMenu>

#include <QDebug>

ContentsSetter::ContentsSetter(const QStringList &userDefinedLists, QWidget *parent)
    : AbstractSetter(parent)
{
    // Layout
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    setLayout(layout);

    // Special contents list
    QList<MenuItem> specialContentsMenuItems;
    specialContentsMenuItems << MenuItem("Matrices", "matrix");
    specialContentsMenuItems << MenuItem("Vectors", "vector");
    specialContentsMenuItems << MenuItem("Row matrices", "row");
    specialContentsMenuItems << MenuItem("Column matrices", "column");
    specialContentsMenuItems << MenuItem("Squared matrices", "square");
    specialContentsMenuItems << MenuItem("Scalars", "scalar");
    specialContentsMenuItems << MenuItem("Constraints", "constraint");
    specialContentsMenuItems << MenuItem("Estimates", "estimates");
    specialContentsMenuItems << MenuItem("Global macros", "global");
    specialContentsMenuItems << MenuItem("Value labels", "valuelabels");

    // List for graphics menu items
    QList<MenuItem> listForGraphicsMenuItems;
    listForGraphicsMenuItems << MenuItem("Marker symbols", "symbols");
    listForGraphicsMenuItems << MenuItem("Marker symbol sizes", "symbolsizes");
    listForGraphicsMenuItems << MenuItem("Colors", "colors");
    listForGraphicsMenuItems << MenuItem("Fill intensity", "intensity");
    listForGraphicsMenuItems << MenuItem("Clock positions", "clockpos");
    listForGraphicsMenuItems << MenuItem("Line patterns", "linepatterns");
    listForGraphicsMenuItems << MenuItem("Line widths", "linewidths");
    listForGraphicsMenuItems << MenuItem("Line connecting types", "connecttypes");
    listForGraphicsMenuItems << MenuItem("Text sizes", "textsizes");
    listForGraphicsMenuItems << MenuItem("Horizontal text justification", "justification");
    listForGraphicsMenuItems << MenuItem("Vectors", "vector");
    listForGraphicsMenuItems << MenuItem("Vertical text alignments", "alignment");
    listForGraphicsMenuItems << MenuItem("Margins", "margins");
    listForGraphicsMenuItems << MenuItem("Axis-tick positions", "tickpos");
    listForGraphicsMenuItems << MenuItem("Angles", "angles");
    listForGraphicsMenuItems << MenuItem("Compass directions", "compass");
    listForGraphicsMenuItems << MenuItem("Yes/No", "yesno");

    // Menus
    QMenu *menu = new QMenu(this);

    // Manual entry
    QAction *manualEntryAction = new QAction("[Manual entry]", this);
    menu->addAction(manualEntryAction);
    connect(manualEntryAction, SIGNAL(triggered()), this, SLOT(onManualEntryActionTriggered()));

    // User defined list menu
    QMenu *userDefinedListMenu = new QMenu("User defined list");
    for (int i=0; i<userDefinedLists.count(); i++)
    {
        QAction *action = new QAction(userDefinedLists.at(i), userDefinedListMenu);
        action->setData(userDefinedLists.at(i));
        userDefinedListMenu->addAction(action);
        connect(action, SIGNAL(triggered()), this, SLOT(onUserDefinedListMenuItemTriggered()));
    }
    menu->addMenu(userDefinedListMenu);

    // Special content menu
    QMenu *specialContentMenu = new QMenu("Special content");
    for (int i=0; i<specialContentsMenuItems.count(); i++)
    {
        QAction *action = new QAction(specialContentsMenuItems.at(i).itemName, specialContentMenu);
        action->setData(specialContentsMenuItems.at(i).itemValue);
        specialContentMenu->addAction(action);
        connect(action, SIGNAL(triggered()), this, SLOT(onMenuActionTriggered()));
    }
    menu->addMenu(specialContentMenu);

    // List for graphics
    QMenu *listForGraphicsMenu = new QMenu("List for graphic");
    for (int i=0; i<listForGraphicsMenuItems.count(); i++)
    {
        QAction *action = new QAction(listForGraphicsMenuItems.at(i).itemName, listForGraphicsMenu);
        action->setData(listForGraphicsMenuItems.at(i).itemValue);
        listForGraphicsMenu->addAction(action);
        connect(action, SIGNAL(triggered()), this, SLOT(onMenuActionTriggered()));
    }
    menu->addMenu(listForGraphicsMenu);

    // Line edit
    lineEdit = new QLineEdit;
    lineEdit->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
    layout->addWidget(lineEdit);

    // Button
    button = new QPushButton;
    button->setMenu(menu);
    button->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    button->setMinimumWidth(20);
    button->setMaximumWidth(20);
    layout->addWidget(button);  
}

void ContentsSetter::onMenuActionTriggered()
{
   QAction *action = qobject_cast<QAction *>(sender());
   lineEdit->hide();
   lineEdit->setText(action->data().toString());
   button->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
   button->setText(action->text());
}

void ContentsSetter::onManualEntryActionTriggered()
{
    lineEdit->show();
    lineEdit->setText("");
    button->setText("");
}

void ContentsSetter::onUserDefinedListMenuItemTriggered()
{
    QAction *action = qobject_cast<QAction *>(sender());
    lineEdit->setText(action->text());
    lineEdit->show();
    button->setText("");
}

void ContentsSetter::setContentsData(const QString &data)
{
    QString name = nameFromValue(data);
    if (!name.isEmpty())
    {
        lineEdit->hide();
        lineEdit->setText(data);
        button->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        button->setText(name);
    }
    else
    {
        lineEdit->show();
        lineEdit->setText(data);
        button->setText("");
    }
}

QString ContentsSetter::nameFromValue(const QString &value)
{
    if (value == "matrix") return "Matrices";
    if (value == "vector") return "Vectors";
    if (value == "row") return "Row matrices";
    if (value == "column") return "Column matrices";
    if (value == "square") return "Squared matrices";
    if (value == "scalar") return "Scalars";
    if (value == "constraint") return "Constraints";
    if (value == "estimates") return "Estimates";
    if (value == "global") return "Global macros";
    if (value == "valuelabels") return "Value labels";
    if (value == "symbols") return "Marker symbols";
    if (value == "symbolsizes") return "Marker symbol sizes";
    if (value == "colors") return "Colors";
    if (value == "intensity") return "Fill intensity";
    if (value == "clockpos") return "Clock positions";
    if (value == "linepatterns") return "Line patterns";
    if (value == "linewidths") return "Line widths";
    if (value == "connecttypes") return "Line connecting types";
    if (value == "textsizes") return "Text sizes";
    if (value == "justification") return "Horizontal text justification";
    if (value == "vector") return "Vectors";
    if (value == "alignment") return "Vertical text alignments";
    if (value == "margins") return "Margins";
    if (value == "tickpos") return "Axis-tick positions";
    if (value == "angles") return "Angles";
    if (value == "compass") return "Compass directions";
    if (value == "yesno") return "Yes/No";
    return "";
}

QString ContentsSetter::contentsData() const
{
    return lineEdit->text();
}
