#include "geometrydataproperty.h"
#include <QDebug>

GeometryDataProperty::GeometryDataProperty(GeometryDataType geometryDataType)
{
    m_geometryDataType = geometryDataType;
}

QVariant GeometryDataProperty::updateDisplayRoleDataString(const QString &valueString)
{
    // valueString should not be empty
    if (valueString.isEmpty())
        return QVariant();

    bool isNumeric;
    valueString.toInt(&isNumeric);
    if (isNumeric)
        return valueString;
    else if (valueString == ".")
        return "Default value";
    else if (valueString == "@")
        return "Previous value";
    else if (!valueString.isEmpty() && valueString.at(0) == '+')
        return valueString;
    else if (!valueString.isEmpty() && valueString.at(0) == '-')
        return valueString;
    else
        return valueString;
}

QString GeometryDataProperty::updateCommandString(const QString &valueString)
{
    return valueString;
}

QString GeometryDataProperty::domElementName() const
{
    switch (m_geometryDataType)
    {
        case GeometryDataProperty::X:
            return "x";
            break;
        case GeometryDataProperty::Y:
            return "y";
            break;
        case GeometryDataProperty::XSize:
            return "xsize";
            break;
        case GeometryDataProperty::YSize:
            return "ysize";
            break;
        default:
            return QString();
            break;
    }
}

QVariant GeometryDataProperty::propertyNameForDisplayRoleData() const
{
    switch (m_geometryDataType)
    {
        case GeometryDataProperty::X:
            return "X";
            break;
        case GeometryDataProperty::Y:
            return "Y";
            break;
        case GeometryDataProperty::XSize:
            return "Width";
            break;
        case GeometryDataProperty::YSize:
            return "Height";
            break;
        default:
            return QVariant();
            break;
    }
}

bool GeometryDataProperty::isRequired() const
{
    return true;
}

AbstractDelegate::DelegateType GeometryDataProperty::delegateForProperty() const
{
    return AbstractDelegate::GeometryData;
}
