#include "objectboxdockwidget.h"
#include <QVBoxLayout>
#include <QHBoxLayout>

ObjectBoxDockWidget::ObjectBoxDockWidget(QWidget *parent) :
    QDockWidget(parent)
{
    setWindowTitle("Object box");
    setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);

    // Inner widget
    QWidget *innerWidget = new QWidget;
    setWidget(innerWidget);

    // Main layout
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setMargin(0);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->setSpacing(0);
    innerWidget->setLayout(mainLayout);

    // Line edit layout
    QHBoxLayout *lineEditLayout = new QHBoxLayout;
    lineEditLayout->setMargin(0);
    lineEditLayout->setContentsMargins(0, 0, 0, 0);
    lineEditLayout->setSpacing(0);
    mainLayout->addLayout(lineEditLayout);

    // Filter line edit
    filterLineEdit = new QLineEdit(this);
    filterLineEdit->setPlaceholderText("<filter>");
    connect(filterLineEdit, SIGNAL(textChanged(QString)), this, SLOT(onFilterLineEditChanged(QString)));
    lineEditLayout->addWidget(filterLineEdit);

    // Clear line edit button
    clearLineEditButton = new QToolButton(this);
    clearLineEditButton->setIcon(QIcon(":images/icons/clearlineedit.png"));
    clearLineEditButton->setDisabled(true);
    lineEditLayout->addWidget(clearLineEditButton);
    connect(clearLineEditButton, SIGNAL(clicked()), this, SLOT(onClearLineEditButtonClicked()));

    // Object box view
    objectBoxView = new ObjectBoxView;
    mainLayout->addWidget(objectBoxView);

    filterLineEdit->clearFocus();
}

void ObjectBoxDockWidget::onFilterLineEditChanged(QString string)
{
    objectBoxView->setFilterString(string);
    if (filterLineEdit->text().isEmpty())
        clearLineEditButton->setDisabled(true);
    else
        clearLineEditButton->setDisabled(false);
}

void ObjectBoxDockWidget::onClearLineEditButtonClicked()
{
    filterLineEdit->setText("");
    filterLineEdit->setPlaceholderText("<filter>");
    filterLineEdit->clearFocus();
}
