#include "objectlistproxymodel.h"

#include "objtypemimedata.h"
#include "model.h"
#include <QDebug>

ObjectListProxyModel::ObjectListProxyModel(QObject *parent) :
    QAbstractProxyModel(parent)
{
}

QModelIndex ObjectListProxyModel::mapFromSource(const QModelIndex &sourceIndex) const
{
    if (!sourceIndex.isValid())
        return QModelIndex();

    // Check for object with no name
    AbstractObj *obj = static_cast<AbstractObj*>(sourceIndex.internalPointer());
    if (obj->hasName() == false && sourceIndex.column() == 1)
        return QModelIndex();

    if (sourceIndex.parent() == QModelIndex())
        return index(sourceIndex.row(), sourceIndex.column(), QModelIndex());
    else
    {
        AbstractObj *obj = static_cast<AbstractObj*>(sourceIndex.internalPointer());
        int parentRow = obj->parentObj()->row();
        QModelIndex parentIndex = index(parentRow, 0, QModelIndex());
        return index(sourceIndex.row(), sourceIndex.column(), parentIndex);
    }
}

QModelIndex ObjectListProxyModel::mapToSource(const QModelIndex &proxyIndex) const
{
    if (!proxyIndex.isValid())
        return QModelIndex();

    // Check for object with no name
    AbstractObj *obj = static_cast<AbstractObj*>(proxyIndex.internalPointer());
    if (obj->hasName() == false && proxyIndex.column() == 1) return QModelIndex();

    if (proxyIndex.parent() == QModelIndex())
        return sourceModel()->index(proxyIndex.row(), proxyIndex.column(), QModelIndex());
    else
    {
        AbstractObj *obj = static_cast<AbstractObj*>(proxyIndex.internalPointer());
        int parentRow = obj->parentObj()->row();
        QModelIndex parentIndex = sourceModel()->index(parentRow, 0, QModelIndex());
        return sourceModel()->index(proxyIndex.row(), proxyIndex.column(), parentIndex);
    }
}

QModelIndex ObjectListProxyModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();

    QModelIndex sourceIndex = sourceModel()->index(row, column, mapToSource(parent));
    AbstractObj* obj = static_cast<AbstractObj*>(sourceIndex.internalPointer());
    return createIndex(row, column, obj);
}

QModelIndex ObjectListProxyModel::parent(const QModelIndex &child) const
{
    if (!child.isValid())
        return QModelIndex();

    AbstractObj *childObj = static_cast<AbstractObj*>(child.internalPointer());
    AbstractObj *parentObj = childObj->parentObj();
    if (parentObj->objType() == AbstractObj::Root)
        return QModelIndex();
    else
        return createIndex(parentObj->row(), 0, parentObj);
}

int ObjectListProxyModel::rowCount(const QModelIndex &parent) const
{
    return sourceModel()->rowCount(mapToSource(parent));
}

int ObjectListProxyModel::columnCount(const QModelIndex &parent) const
{
    return 2;
}

QVariant ObjectListProxyModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    switch (role)
    {
        case Qt::DisplayRole:
            return sourceModel()->data(mapToSource(index), Qt::DisplayRole);
            break;
        case Qt::EditRole:
            return QVariant();
            break;
        case Qt::DecorationRole:
            {
            if (index.isValid() && index.column() == 0)
            {
                AbstractObj *obj = static_cast<AbstractObj*>(index.internalPointer());
                AbstractObj::ObjType objType = obj->objType();
                switch (objType)
                {
                    case AbstractObj::Version:
                        return QIcon(":images/icons/objicons/version.png");
                        break;
                    case AbstractObj::Include:
                        return QIcon(":images/icons/objicons/include.png");
                        break;
                    case AbstractObj::Define:
                        return QIcon(":images/icons/objicons/define.png");
                        break;
                    case AbstractObj::Position:
                        return QIcon(":images/icons/objicons/position.png");
                        break;
                    case AbstractObj::List:
                        return QIcon(":images/icons/objicons/list.png");
                        break;
                    case AbstractObj::Dialog:
                        return QIcon(":images/icons/objicons/dialog.png");
                        break;
                    case AbstractObj::CheckBox:
                        return QIcon(":images/icons/objicons/checkbox.png");
                        break;
                    case AbstractObj::Radio:
                        return QIcon(":images/icons/objicons/radio.png");
                        break;
                    case AbstractObj::Spinner:
                        return QIcon(":images/icons/objicons/spinner.png");
                        break;
                    case AbstractObj::Edit:
                        return QIcon(":images/icons/objicons/edit.png");
                        break;
                    case AbstractObj::VarList:
                        return QIcon(":images/icons/objicons/varlist.png");
                        break;
                    case AbstractObj::VarName:
                        return QIcon(":images/icons/objicons/varname.png");
                        break;
                    case AbstractObj::File:
                        return QIcon(":images/icons/objicons/file.png");
                        break;
                    case AbstractObj::ListBox:
                        return QIcon(":images/icons/objicons/listbox.png");
                        break;
                    case AbstractObj::ComboBox:
                        return QIcon(":images/icons/objicons/combobox.png");
                        break;
                    case AbstractObj::Button:
                        return QIcon(":images/icons/objicons/button.png");
                        break;
                    case AbstractObj::Text:
                        return QIcon(":images/icons/objicons/text.png");
                        break;
                    case AbstractObj::TextBox:
                        return QIcon(":images/icons/objicons/textbox.png");
                        break;
                    case AbstractObj::GroupBox:
                        return QIcon(":images/icons/objicons/groupbox.png");
                        break;
                    case AbstractObj::Frame:
                        return QIcon(":images/icons/objicons/frame.png");
                        break;
                    case AbstractObj::Color:
                        return QIcon(":images/icons/objicons/color.png");
                        break;
                    case AbstractObj::Exp:
                        return QIcon(":images/icons/objicons/exp.png");
                        break;
                    case AbstractObj::Ok:
                        return QIcon(":images/icons/objicons/ok.png");
                        break;
                    case AbstractObj::Submit:
                        return QIcon(":images/icons/objicons/submit.png");
                        break;
                    case AbstractObj::Cancel:
                        return QIcon(":images/icons/objicons/cancel.png");
                        break;
                    case AbstractObj::Copy:
                        return QIcon(":images/icons/objicons/copy.png");
                        break;
                    case AbstractObj::Help:
                        return QIcon(":images/icons/objicons/help.png");
                        break;
                    case AbstractObj::Reset:
                        return QIcon(":images/icons/objicons/reset.png");
                        break;
                    case AbstractObj::Script:
                        return QIcon(":images/icons/objicons/script.png");
                        break;
                    case AbstractObj::Program:
                        return QIcon(":images/icons/objicons/program.png");
                        break;
                    default:
                        return QIcon(":images/icons/test16x16.png");
                        break;
                }
            }
            else
                return QVariant();
            }
            break;

        default:
            return QVariant();
            break;
    }
}

QVariant ObjectListProxyModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    // Header data is provided only for display role
    if (role != Qt::DisplayRole)
        return QVariant();

    switch (orientation)
    {
        case Qt::Horizontal:
            switch (section)
            {
                case 0:
                    return "Type";
                case 1:
                    return "Name";
                default:
                    return QVariant();
            }
            break;
        case Qt::Vertical:
            return QVariant();
            break;
        default:
            return QVariant();
            break;
    }
}

Qt::ItemFlags ObjectListProxyModel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return QAbstractItemModel::flags(index) | Qt::ItemIsDropEnabled;

    switch (index.column())
    {
        case 0: // Column "Type"
            return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDropEnabled;
            break;
        case 1: // Column "Name"
            return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDropEnabled;
            break;
        default:
            return QAbstractItemModel::flags(index) | Qt::ItemIsDropEnabled;
            break;
    }
}

bool ObjectListProxyModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    return true;
}

void ObjectListProxyModel::onMainModelWasResetted()
{
    reset();
}

bool ObjectListProxyModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
                  int row, int column, const QModelIndex &referenceIndex)
{
    Q_UNUSED(action)
    const ObjTypeMimeData *objTypeMimeData =
            qobject_cast<const ObjTypeMimeData*>(data);
    if (objTypeMimeData)
    {
        Model *model = static_cast<Model*>(sourceModel());
        AbstractObj::ObjType objType = objTypeMimeData->objType();
        if (row == -1 && column == -1) // Drop inside item
        {
            if (referenceIndex.isValid()) // Check if reference is valid
            {
                AbstractObj *receivingObj = static_cast<AbstractObj*>(referenceIndex.internalPointer());
                AbstractObj::ObjType receivingObjType = receivingObj->objType();
                if (receivingObjType == AbstractObj::Dialog) // Receiving item is a dialog
                {
                    if (!objCanBeDroppedInsideDialog(objType))
                    {
                        emit wasAttemptedToInsertProhibitedObjInDialog();
                        return false;
                    }

                    QModelIndex indexAtColZero = index(referenceIndex.row(), 0, referenceIndex.parent());
                    emit layoutAboutToBeChanged();
                    model->appendNewObj(objType, mapToSource(indexAtColZero));
                    emit layoutChanged();

                    int childCount = rowCount(referenceIndex);
                    QModelIndex newIndex = index(childCount-1, 0, referenceIndex);
                    emit newItemAppended(newIndex);

                    return true;
                }
                else // Receiving item is not a dialog
                {
                    QModelIndex indexAtColZero = index(referenceIndex.row(), 0, referenceIndex.parent());

                    QModelIndex parentIndex = mapToSource(indexAtColZero.parent());
                    if (parentIndex != QModelIndex())
                    {
                        AbstractObj *parentObj = static_cast<AbstractObj*>(parentIndex.internalPointer());
                        if (parentObj->objType() == AbstractObj::Dialog &&
                            !objCanBeDroppedInsideDialog(objType))
                        {
                            emit wasAttemptedToInsertProhibitedObjInDialog();
                            return false;
                        }
                    }

                    emit layoutAboutToBeChanged();
                    model->insertNewObj(objType, parentIndex, referenceIndex.row() + 1);
                    emit layoutChanged();

                    QModelIndex newIndex = index(referenceIndex.row() + 1, 0, indexAtColZero.parent());
                    emit newItemAppended(newIndex);

                    return true;
                }
            }
            else // Drop on blank part of view
            {
                emit layoutAboutToBeChanged();
                model->appendNewObj(objType, QModelIndex());
                emit layoutChanged();

                int nItems = rowCount();
                QModelIndex newIndex = index(nItems-1, 0);
                emit newItemAppended(newIndex);

                return true;
            }
        }
        else // Drop between items
        {
            QModelIndex indexAtColZero = index(referenceIndex.row(), 0, referenceIndex.parent());

            QModelIndex parentIndex = mapToSource(indexAtColZero);
            if (parentIndex != QModelIndex())
            {
                AbstractObj *parentObj = static_cast<AbstractObj*>(parentIndex.internalPointer());
                if (parentObj->objType() == AbstractObj::Dialog &&
                        !objCanBeDroppedInsideDialog(objType))
                {
                    emit wasAttemptedToInsertProhibitedObjInDialog();
                    return false;
                }
            }

            emit layoutAboutToBeChanged();
            model->insertNewObj(objType, mapToSource(indexAtColZero), row);
            emit layoutChanged();

            QModelIndex newIndex = index(row, 0, referenceIndex);
            emit newItemAppended(newIndex);

            return true;
        }
    }
    else
        return false;
}

QStringList ObjectListProxyModel::mimeTypes() const
{
    QStringList types;
    types << "text/plain";
    return types;
}

Qt::DropActions ObjectListProxyModel::supportedDropActions() const
{
    return Qt::CopyAction;
}

bool ObjectListProxyModel::moveItemUp(const QModelIndex &parent, int row)
{
    Model *model = static_cast<Model*>(sourceModel());
    beginMoveRows(parent, row, row, parent, row-1);
    model->moveItemUp(parent, row);
    endMoveRows();
    return true;
}

bool ObjectListProxyModel::moveItemDown(const QModelIndex &parent, int row)
{
    Model *model = static_cast<Model*>(sourceModel());
    beginMoveRows(parent, row, row, parent, row+2);
    model->moveItemDown(parent, row);
    endMoveRows();
    return true;
}

bool ObjectListProxyModel::removeRows(int row, int count, const QModelIndex &parent)
{
    emit layoutAboutToBeChanged();
    beginRemoveRows(parent, row, row + count - 1);
    sourceModel()->removeRows(row, count, mapToSource(parent));
    endRemoveRows();
    emit layoutChanged();
    return true;
}

bool ObjectListProxyModel::objCanBeDroppedInsideDialog(AbstractObj::ObjType objType)
{
    switch (objType)
    {
        case AbstractObj::CheckBox:
        case AbstractObj::Radio:
        case AbstractObj::Spinner:
        case AbstractObj::Edit:
        case AbstractObj::VarList:
        case AbstractObj::VarName:
        case AbstractObj::File:
        case AbstractObj::ListBox:
        case AbstractObj::ComboBox:
        case AbstractObj::Button:
        case AbstractObj::Text:
        case AbstractObj::TextBox:
        case AbstractObj::GroupBox:
        case AbstractObj::Frame:
        case AbstractObj::Color:
        case AbstractObj::Exp:
            return true;
            break;
        default:
            return false;
            break;
    }
}

