#include "positiongeometrydataproperty.h"

PositionGeometryDataProperty::PositionGeometryDataProperty(GeometryDataType geometryDataType,
                                                           bool defaultValueAllowed)
{
    m_geometryDataType = geometryDataType;
    m_defaultValueAllowed = defaultValueAllowed;
}

QVariant PositionGeometryDataProperty::updateDisplayRoleDataString(const QString &valueString)
{
    // valueString should not be empty
    if (valueString.isEmpty())
        return QVariant();

    bool isNumeric;
    valueString.toInt(&isNumeric);
    if (isNumeric)
        return valueString;
    else if (valueString == ".")
        return "Default value";
    else
        return valueString;
}

QString PositionGeometryDataProperty::updateCommandString(const QString &valueString)
{
    return valueString;
}

QString PositionGeometryDataProperty::domElementName() const
{
    switch (m_geometryDataType)
    {
        case PositionGeometryDataProperty::X:
            return "x";
            break;
        case PositionGeometryDataProperty::Y:
            return "y";
            break;
        case PositionGeometryDataProperty::XSize:
            return "xsize";
            break;
        case PositionGeometryDataProperty::YSize:
            return "ysize";
            break;
        default:
            return QString();
            break;
    }
}

QVariant PositionGeometryDataProperty::propertyNameForDisplayRoleData() const
{
    switch (m_geometryDataType)
    {
        case PositionGeometryDataProperty::X:
            return "X";
            break;
        case PositionGeometryDataProperty::Y:
            return "Y";
            break;
        case PositionGeometryDataProperty::XSize:
            return "Width";
            break;
        case PositionGeometryDataProperty::YSize:
            return "Height";
            break;
        default:
            return QVariant();
            break;
    }
}

bool PositionGeometryDataProperty::isRequired() const
{
    return true;
}

AbstractDelegate::DelegateType PositionGeometryDataProperty::delegateForProperty() const
{
    if (m_defaultValueAllowed)
        return AbstractDelegate::PositionGeometryXYData;
    else
        return AbstractDelegate::PositionGeometryXSizeYSizeData;
}
