#include "statalogview.h"
#include <QFile>
#include <QTextStream>

StataLogView::StataLogView(QWidget *parent) :
    QMainWindow(parent)
{
    setAttribute(Qt::WA_GroupLeader);
    setWindowFlags(Qt::Window);
    setWindowTitle("Visua - Last Stata log file");

    // Text edit
    textEdit = new QTextEdit(this);
    textEdit->setReadOnly(true);
    textEdit->setFont(QFont("Lucida console", 10, QFont::Normal));
    setCentralWidget(textEdit);

    createActions();
    createMenus();
}

bool StataLogView::loadLastLogFile()
{
    // "Visuatest" dir
    QDir visuaTestDir(QApplication::applicationDirPath());
    bool visuaTestDirWasFound = visuaTestDir.cd("Visuatest");
    if (!visuaTestDirWasFound)
    {
        QMessageBox::critical(this,
                              "Visua",
                              "The \"Visuatest\" folder was not found");
        return false;
    }
    QString visuaTestAbsolutePath = visuaTestDir.absolutePath();
    QString visuaTestLogFile = visuaTestAbsolutePath + QString("/visuatest.log");

    QFile stataLogFile(visuaTestLogFile);
    if (!stataLogFile.open(QIODevice::ReadOnly | QIODevice::Text))
        return false;

    QString stataLogText;
    QTextStream stataLogFileTextStream(&stataLogFile);

    while (!stataLogFileTextStream.atEnd())
        stataLogText += stataLogFileTextStream.readLine() + "\n";

    textEdit->setText(stataLogText);

    stataLogFile.close();
    return true;
}

void StataLogView::createActions()
{
    // Save as...
    saveAsAction = new QAction("Save as...", this);
    connect(saveAsAction, SIGNAL(triggered()), this, SLOT(onSaveAsActionTriggered()));

    // Exit
    exitAction = new QAction("Exit", this);
    connect(exitAction, SIGNAL(triggered()), this, SLOT(close()));

    // Update
    updateAction = new QAction("Update", this);
    connect(updateAction, SIGNAL(triggered()), this, SLOT(onUpdateActionTriggered()));

    // Show path of log file
    showPathOfLogFileAction = new QAction("Show path of log file", this);
    connect(showPathOfLogFileAction, SIGNAL(triggered()), this, SLOT(onShowPathOfLogActionTriggered()));
}

void StataLogView::createMenus()
{
    // File menu
    fileMenu = menuBar()->addMenu("&File");
    fileMenu->addAction(saveAsAction);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAction);

    // Log file menu
    logFileMenu = menuBar()->addMenu("&Log file");
    logFileMenu->addAction(updateAction);
    logFileMenu->addAction(showPathOfLogFileAction);
}

void StataLogView::onUpdateActionTriggered()
{
    if (!loadLastLogFile())
        QMessageBox::warning(
            this,
            "Visua",
            "Last log file was not found or it was not possible to open the log file",
            QMessageBox::Ok);
}

void StataLogView::onShowPathOfLogActionTriggered()
{
    QMessageBox::information(this,
                             "Visua",
                             QString("The last log file is in the following position:\n%1/visuatest.log")
                             .arg(QApplication::applicationDirPath()));
}

void StataLogView::onSaveAsActionTriggered()
{
    QString fileName = QFileDialog::getSaveFileName(
                this,
                "Save Stata log file",
                ".",
                "Visua files (*.log)");
    if (!fileName.isEmpty())
    {
        QFile file(fileName);
        if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
        {
            QMessageBox::critical(
                        this,
                        "Visua",
                        "Error saving log file");
            return;
        }
        else
        {
            QTextStream textStream(&file);
            textStream << textEdit->document()->toPlainText();
        }
    }
}
