#include "uactionsetter.h"
#include <QHBoxLayout>
#include <QAction>
#include <QMenu>
#include <QRegExpValidator>

UActionSetter::UActionSetter(QWidget *parent, const QStringList &programList)
    : AbstractSetter(parent)
{
    createMenu(programList);

    // Layout
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    setLayout(layout);

    // Completer
    QCompleter *completer = new QCompleter(programList, this);
    completer->setCaseSensitivity(Qt::CaseInsensitive);

    // Line edit
    lineEdit = new QLineEdit;
    QRegExp regExp("[^\\d][\\w]+");
    QRegExpValidator *regExpValidator = new QRegExpValidator(regExp, lineEdit);
    lineEdit->setValidator(regExpValidator);
    lineEdit->setCompleter(completer);
    lineEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    lineEdit->setFocus();
    lineEdit->setCursorPosition(1);
    layout->addWidget(lineEdit);

    // Button
    button = new QPushButton;
    button->setMenu(mainMenu);
    button->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    button->setMinimumWidth(20);
    button->setMaximumWidth(20);
    layout->addWidget(button);
}

void UActionSetter::setUActionData(const QString &data)
{
    lineEdit->setText(data);
}

QString UActionSetter::uActionData()
{
    return lineEdit->text();
}

void UActionSetter::createMenu(const QStringList &programList)
{
    // Main menu
    mainMenu = new QMenu(this);

    // Program Menu
    QMenu *programMenu = new QMenu("Program", this);
    mainMenu->addMenu(programMenu);

    // Separator
    mainMenu->addSeparator();

    // Clear action
    QAction *clearAction = new QAction("Clear", this);
    connect(clearAction, SIGNAL(triggered()), this, SLOT(onClearActionTriggered()));
    mainMenu->addAction(clearAction);

    if (programList.isEmpty())
        programMenu->setDisabled(true);

    for (int i=0; i<programList.count(); i++)
    {
        QAction *action = new QAction(programList.at(i), this);
        action->setData(programList.at(i));
        connect(action, SIGNAL(triggered()), this, SLOT(onProgramChosenFromMenu()));
        programMenu->addAction(action);
    }
}

void UActionSetter::onProgramChosenFromMenu()
{
    QAction *action = qobject_cast<QAction *>(sender());
    QString text = action->data().toString();
    lineEdit->setText(text);
}

void UActionSetter::onClearActionTriggered()
{
    lineEdit->setText("");
}
