#include "valuessetter.h"
#include <QHBoxLayout>
#include <QMenu>
#include <QRegExpValidator>


ValuesSetter::ValuesSetter(const QStringList &userDefinedLists, QWidget *parent)
    : AbstractSetter(parent)
{
    // Layout
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    setLayout(layout);

    // Menu
    QMenu *menu = new QMenu(this);

    // User defined list menu
    QMenu *userDefinedListMenu = new QMenu("User defined list");
    for (int i=0; i<userDefinedLists.count(); i++)
    {
        QAction *action = new QAction(userDefinedLists.at(i), userDefinedListMenu);
        action->setData(userDefinedLists.at(i));
        userDefinedListMenu->addAction(action);
        connect(action, SIGNAL(triggered()), this, SLOT(onUserDefinedListMenuItemTriggered()));
    }
    if (userDefinedLists.count() == 0) userDefinedListMenu->setDisabled(true);
    menu->addMenu(userDefinedListMenu);

    // Clear action
    QAction *clearAction = new QAction("Clear", this);
    menu->addAction(clearAction);
    connect(clearAction, SIGNAL(triggered()), this, SLOT(onClearActionsTriggered()));

    // Line edit
    lineEdit = new QLineEdit;
    lineEdit->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
    lineEdit->setMaxLength(32);
    QRegExp regExp("[^\\d][\\w]+");
    QRegExpValidator *regExpValidator = new QRegExpValidator(regExp, lineEdit);
    lineEdit->setValidator(regExpValidator);
    layout->addWidget(lineEdit);

    // Button
    button = new QPushButton;
    button->setMenu(menu);
    button->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    button->setMinimumWidth(20);
    button->setMaximumWidth(20);
    layout->addWidget(button);
}

void ValuesSetter::setValuesData(const QString &data)
{
    lineEdit->setText(data);
}

QString ValuesSetter::valuesData() const
{
    return lineEdit->text();
}

void ValuesSetter::onUserDefinedListMenuItemTriggered()
{
    QAction *action = qobject_cast<QAction *>(sender());
    lineEdit->setText(action->text());
}

void ValuesSetter::onClearActionsTriggered()
{
    lineEdit->clear();
}
